/**
 * JMainMenu.java, the main menu for JamochaMUD made of Swing components
 * $Id: JMainMenu.java,v 1.5 2004/07/28 03:13:05 jeffnik Exp $
 */

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2004  Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * vesion 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package anecho.JamochaMUD;

// import java.awt.MenuShortcut;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import javax.swing.KeyStroke;

import javax.swing.ImageIcon;

import anecho.gui.ResReader;

/** Create a menu for the Main Window */
public class JMainMenu {

    private static JMainMenu _instance;  // We'll try playing with a Singleton!

    private JMenuBar tWMenuBar;
    private JCheckBoxMenuItem tWMacro, tWSyncWindowsItem, tWUseUnicodeItem, tWTFKeysItem, tWLocalEchoItem;
    // private JCheckBoxMenuItem tWDoubleBufferItem;
    private JCheckBoxMenuItem splitFramesItem;
    private JCheckBoxMenuItem tWAutoFocus, tWTimers, tWReleasePauseItem, tWAltFocus, tWAntiAliasItem;
    private JMenu tWMUListMenu;
    // private JMenuItem tempMenuItem;
    private JMenuItem cTM, dFM, rTM, closeMU;
    private JMenuItem dumpOutputItem, quitItem;
    private JMenuItem pingMUItem;
    private JMenuItem copyItem, pasteItem, findItem;
    private JMenuItem coloursItem, externalProgramsItem;
    // private JMenuItem managePlugIn;
    private JMenuItem serverOptionsItem;
    private JMenuItem contentsItem, tfKeysItem, troubleshootingItem, aboutJamochaMUDItem;
    private JMenuItem installPlugIn, removePlugIn;
    private JMConfig settings;
    protected static JMenu tWPlugInMenu;

    private static final int COMBINED = 0;     // Combined frame style
    private static final int SPLIT = 1;        // Split frame style

    private int viewStyle;

    private ResReader reader;

    private static final boolean DEBUG = false;

    private JMainMenu() {
        reader = new ResReader();
    }

    // For lazy initialization
    public static synchronized JMainMenu getInstance() {
        if (_instance==null) {
            _instance = new JMainMenu();
        }
        return _instance;
    }

    public void buildMenu(JFrame parent, MuckMain mainProg, JMConfig settings) {
        this.settings = settings;
        tWMenuBar = new JMenuBar();

        parent.setJMenuBar(tWMenuBar);

        JMenu tWFileMenu = new JMenu(RB("file"));
        tWFileMenu.setMnemonic(KeyEvent.VK_F);
        tWMenuBar.add(tWFileMenu);

        dumpOutputItem = new JMenuItem(RB("dumpOutput"), new ImageIcon(JMainMenu.class.getResource("icons/22/filesave.png")));
        dumpOutputItem.setMnemonic(KeyEvent.VK_D);
        dumpOutputItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        // dumpOutputItem = buildMenuItem("dumpOutput",            // Name of item
        // "icons/22/filesave.png", // Item icon
        // KeyEvent.VK_D,           // Mnemonic
        // true,                    // Has an accelerator?
        // KeyEvent.VK_D,           // Accelerator key
        // true);                   // use CTRL?

        tWFileMenu.add(dumpOutputItem);
        // new MenuShortcut(KeyEvent.VK_D, false)));
        dumpOutputItem.setActionCommand(dumpOutputItem.getText());
        dumpOutputItem.addActionListener(mainProg);

        tWFileMenu.addSeparator();
        tWFileMenu.add(quitItem = new JMenuItem(RB("quit"), new ImageIcon(JMainMenu.class.getResource("icons/22/kill.png"))));
        quitItem.setMnemonic(KeyEvent.VK_Q);
        quitItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_Q, true)));
        quitItem.setActionCommand(quitItem.getText());
        quitItem.addActionListener(mainProg);

        tWFileMenu.addActionListener(mainProg);

        // Add the Edit menu items
        JMenu tWEditMenu = new JMenu(RB("edit"));
        tWEditMenu.setMnemonic(KeyEvent.VK_E);
        tWMenuBar.add(tWEditMenu);
        // I'm making it the right key for OS/2, damn't!
        // if (settings.getOSName().equals("OS/2")) {
        if ((settings.getJMString(JMConfig.OSNAME)).equals("OS/2")) {
        // if (settings.getJMString(settings.OSNAME).equals("OS/2")) {
            copyItem = new JMenuItem(RB("copyFromMainWindow"), new ImageIcon(JMainMenu.class.getResource("icons/22/editcopy.png")));
            copyItem.setMnemonic(KeyEvent.VK_C);
            copyItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, ActionEvent.CTRL_MASK));
            // new MenuShortcut(KeyEvent.VK_INSERT, false));
        } else {
            copyItem = new JMenuItem(RB("copyFromMainWindow"), new ImageIcon(JMainMenu.class.getResource("icons/22/editcopy.png")));
            copyItem.setMnemonic(KeyEvent.VK_C);
            copyItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
            // new MenuShortcut(KeyEvent.VK_C, false));
        }
        copyItem.setActionCommand(copyItem.getText());
        copyItem.addActionListener(mainProg);
        tWEditMenu.add(copyItem);

        pasteItem = new JMenuItem(RB("paste"), new ImageIcon(JMainMenu.class.getResource("icons/22/editpaste.png")));
        pasteItem.setMnemonic(KeyEvent.VK_P);
        pasteItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_V, false));
        pasteItem.setActionCommand(pasteItem.getText());
        pasteItem.addActionListener(mainProg);
        tWEditMenu.add(pasteItem);

        tWEditMenu.addSeparator();

        findItem = new JMenuItem(RB("find"), new ImageIcon(JMainMenu.class.getResource("icons/22/find.png")));
        findItem.setMnemonic(KeyEvent.VK_F);
        findItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_F, false));
        findItem.setActionCommand(findItem.getText());
        findItem.addActionListener(mainProg);
        findItem.setEnabled(false);
        tWEditMenu.add(findItem);
        tWEditMenu.addActionListener(mainProg);


        // Add CONNECTION menu items
        JMenu tWConnectMenu = new JMenu(RB("connection"));
        tWConnectMenu.setMnemonic(KeyEvent.VK_C);
        tWMenuBar.add(tWConnectMenu);
        cTM = new JMenuItem(RB("connectToMU"), new ImageIcon(JMainMenu.class.getResource("icons/22/connect_established.png")));
        cTM.setMnemonic(KeyEvent.VK_C);
        cTM.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_C, true)); // enable/disable functionality
        cTM.setActionCommand(cTM.getText());
        cTM.addActionListener(mainProg);
        tWConnectMenu.add(cTM);

        rTM = new JMenuItem(RB("reconnectToMU"), new ImageIcon(JMainMenu.class.getResource("icons/22/reload.png")));
        rTM.setMnemonic(KeyEvent.VK_R);
        rTM.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_R, true));
        rTM.setActionCommand(rTM.getText());
        rTM.addActionListener(mainProg);
        tWConnectMenu.add(rTM);

        dFM = new JMenuItem(RB("disconnectFromMU"), new ImageIcon(JMainMenu.class.getResource("icons/22/connect_no.png")));
        dFM.setMnemonic(KeyEvent.VK_D);
        dFM.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_D, true));  // enable/disable functionality
        dFM.setActionCommand(dFM.getText());
        dFM.addActionListener(mainProg);
        dFM.setEnabled(false);
        tWConnectMenu.add(dFM);

        closeMU = new JMenuItem(RB("closeThisView"), new ImageIcon(JMainMenu.class.getResource("icons/22/round_stop.png")));
        closeMU.setMnemonic(KeyEvent.VK_V);
        closeMU.addActionListener(mainProg);
        closeMU.setEnabled(false);
        tWConnectMenu.add(closeMU);

        tWConnectMenu.add(pingMUItem = new JMenuItem(RB("pingMU")));
        pingMUItem.addActionListener(mainProg);
        pingMUItem.setEnabled(false);
        tWConnectMenu.addActionListener(mainProg);

        JMenu tWOptionMenu = new JMenu(RB("options"));
        tWOptionMenu.setMnemonic(KeyEvent.VK_O);
        tWMenuBar.add(tWOptionMenu);

        // Add ****Edit Options**** Submenu
        JMenu tWEditOptionsMenu = new JMenu(RB("editOptions"));
        tWEditOptionsMenu.setMnemonic(KeyEvent.VK_E);
        tWOptionMenu.add(tWEditOptionsMenu);
        tWEditOptionsMenu.add(externalProgramsItem = new JMenuItem(RB("externalPrograms"), new ImageIcon(JMainMenu.class.getResource("icons/22/configure.png"))));
        externalProgramsItem.setMnemonic(KeyEvent.VK_E);
        externalProgramsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_E, false)));
        externalProgramsItem.setActionCommand(externalProgramsItem.getText());
        externalProgramsItem.addActionListener(mainProg);
        // externalProgramsItem.setEnabled(false);

        /*
        tWEditOptionsMenu.add(managePlugIn = new JMenuItem(RB("ManagePlugins.title"), new ImageIcon(JMainMenu.class.getResource("icons/22/configure.png"))));
        managePlugIn.setMnemonic(KeyEvent.VK_M);
        managePlugIn.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_M, false)));
        managePlugIn.setActionCommand(managePlugIn.getText());
        managePlugIn.addActionListener(mainProg);
        */

        installPlugIn = new JMenuItem(RB("installPlugin"), new ImageIcon(JMainMenu.class.getResource("icons/22/fileexport.png")));
        installPlugIn.setMnemonic(KeyEvent.VK_I);
        // pasteItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        installPlugIn.setActionCommand(installPlugIn.getText());
        installPlugIn.addActionListener(mainProg);

        removePlugIn = new JMenuItem(RB("removePlugin"));
        removePlugIn.setMnemonic(KeyEvent.VK_R);
        // pasteItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        removePlugIn.setEnabled(false);
        removePlugIn.setActionCommand(removePlugIn.getText());
        removePlugIn.addActionListener(mainProg);

        tWEditOptionsMenu.add(serverOptionsItem = new JMenuItem(RB("serverOptions"), new ImageIcon(JMainMenu.class.getResource("icons/22/configure.png"))));
        serverOptionsItem.setMnemonic(KeyEvent.VK_S);
        tWEditOptionsMenu.addActionListener(mainProg);
        serverOptionsItem.addActionListener(mainProg);

        JMenu tWDisplayOptionsMenu = new JMenu(RB("displayOptions"));
        tWOptionMenu.add(tWDisplayOptionsMenu);

        // tWOptionMenu.add(coloursItem = new JMenuItem(RB("fontsAndColours")));
        tWDisplayOptionsMenu.add(coloursItem = new JMenuItem(RB("fontsAndColours")));
        coloursItem.setMnemonic(KeyEvent.VK_F);
        coloursItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
        // new MenuShortcut(KeyEvent.VK_F, true)));
        coloursItem.setActionCommand(coloursItem.getText());
        coloursItem.addActionListener(mainProg);

        tWAntiAliasItem = new JCheckBoxMenuItem(RB("antiAlias"), true);
        tWDisplayOptionsMenu.add(tWAntiAliasItem);
        tWAntiAliasItem.setState(settings.getJMboolean(JMConfig.ANTIALIAS));
        tWAntiAliasItem.addItemListener(mainProg);

        /*
        tWDoubleBufferItem = new JCheckBoxMenuItem(RB("doubleBuffer"), true);
        tWDisplayOptionsMenu.add(tWDoubleBufferItem);
        // tWDoubleBufferItem.setState(settings.getDoubleBuffer());
        tWDoubleBufferItem.setState(settings.getJMboolean(JMConfig.DOUBLEBUFFER));
        tWDoubleBufferItem.addItemListener(mainProg);
        */

        // Continue with normal option menu
        tWOptionMenu.addSeparator();

        tWMacro = new JCheckBoxMenuItem(RB("showMacroBar"), false);
        tWOptionMenu.add(tWMacro);
        tWMacro.addItemListener(mainProg);
        tWMacro.setEnabled(false);

        tWTimers = new JCheckBoxMenuItem(RB("showTimers"), false);
        tWTimers.setMnemonic(KeyEvent.VK_T);
        tWTimers.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
        // tWTimers.setShortcut(new MenuShortcut(KeyEvent.VK_T, false));
        tWTimers.setActionCommand(tWTimers.getText());
        tWOptionMenu.add(tWTimers);
        tWTimers.addItemListener(mainProg);
        tWTimers.addActionListener(mainProg);
        // Fix mainProg XXX!!  Right now we have no option for timers at all
        tWTimers.setEnabled(false);

        splitFramesItem = new JCheckBoxMenuItem(RB("splitFrames"), false);
        tWOptionMenu.add(splitFramesItem);
        // if (settings.getSplitView()) {
        if (settings.getJMboolean(JMConfig.SPLITVIEW)) {
            viewStyle = SPLIT;
            splitFramesItem.setState(true);
        } else {
            viewStyle = COMBINED;
        }
        splitFramesItem.addItemListener(mainProg);

        tWSyncWindowsItem = new JCheckBoxMenuItem(RB("syncWindows"), true);
        tWOptionMenu.add(tWSyncWindowsItem);
        // tWSyncWindowsItem.setState(settings.getSyncWindows());
        tWSyncWindowsItem.setState(settings.getJMboolean(JMConfig.SYNCWINDOWS));
        tWSyncWindowsItem.addItemListener(mainProg);

        tWOptionMenu.addSeparator();

        tWAutoFocus = new JCheckBoxMenuItem(RB("autoFocusInput"), true);
        tWOptionMenu.add(tWAutoFocus);
        // tWAutoFocus.setState(settings.getAutoFocusInput());
        tWAutoFocus.setState(settings.getJMboolean(settings.AUTOFOCUSINPUT));
        tWAutoFocus.addItemListener(mainProg);

        tWAltFocus = new JCheckBoxMenuItem(RB("altFocus"), true);
        tWOptionMenu.add(tWAltFocus);
        // tWAltFocus.setState(settings.getAltFocus());
        tWAltFocus.setState(settings.getJMboolean(JMConfig.ALTFOCUS));
        if (!splitFramesItem.getState()) {
            tWAltFocus.setEnabled(false);
        }
        tWAltFocus.addItemListener(mainProg);

        tWLocalEchoItem = new JCheckBoxMenuItem(RB("localEcho"), true);
        tWOptionMenu.add(tWLocalEchoItem);
        // tWLocalEchoItem.addActionListener(mainProg);
        // tWLocalEchoItem.setState(settings.isLocalEchoEnabled());
        tWLocalEchoItem.setState(settings.getJMboolean(JMConfig.LOCALECHO));
        tWLocalEchoItem.addItemListener(mainProg);

        tWReleasePauseItem = new JCheckBoxMenuItem(RB("releasePause"), true);
        tWOptionMenu.add(tWReleasePauseItem);
        // tWReleasePauseItem.setState(settings.getReleasePause());
        tWReleasePauseItem.setState(settings.getJMboolean(JMConfig.RELEASEPAUSE));
        tWReleasePauseItem.addItemListener(mainProg);

        tWTFKeysItem = new JCheckBoxMenuItem(RB("tinyFugueKeys"), true);
        tWOptionMenu.add(tWTFKeysItem);
        // tWTFKeysItem.setState(settings.getTFKeyEmu());
        tWTFKeysItem.setState(settings.getJMboolean(settings.TFKEYEMU));
        tWTFKeysItem.addItemListener(mainProg);

        tWUseUnicodeItem = new JCheckBoxMenuItem(RB("useUnicode"), false);
        tWOptionMenu.add(tWUseUnicodeItem);
        // tWUseUnicodeItem.setState(settings.getUseUnicode());
        tWUseUnicodeItem.setState(settings.getJMboolean(settings.USEUNICODE));
        tWUseUnicodeItem.addItemListener(mainProg);

        // Add the ActionListener to mainProg menu
        tWOptionMenu.addActionListener(mainProg);


        // Add the Plug-in menu items
        tWPlugInMenu = new JMenu(RB("plugIn"));
        tWPlugInMenu.setMnemonic(KeyEvent.VK_P);
        tWMenuBar.add(tWPlugInMenu);
        tWPlugInMenu.addActionListener(mainProg);

        // List for our active MU*s
        tWMUListMenu = new JMenu(RB("MUList"));
        tWMUListMenu.setMnemonic(KeyEvent.VK_M);
        tWMenuBar.add(tWMUListMenu);
        tWMUListMenu.addActionListener(mainProg);

        //Add HELP menu items
        JMenu tWHelpMenu = new JMenu(RB("help"));
        tWHelpMenu.setMnemonic(KeyEvent.VK_H);
        // tWMenuBar.setHelpMenu(tWHelpMenu);
        tWMenuBar.add(tWHelpMenu);
        // tWHelpMenu.add(contentsItem = new JMenuItem(RB("contents"), (MenuShortcut)new MenuShortcut(KeyEvent.VK_H)));
        tWHelpMenu.add(contentsItem = new JMenuItem(RB("contents"), new ImageIcon(JMainMenu.class.getResource("icons/22/contents.png"))));
        contentsItem.setMnemonic(KeyEvent.VK_H);
        contentsItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
        // tWHelpMenu.setShortCut(new MenuShortcut(KeyEvent.VK_H));
        contentsItem.setActionCommand(contentsItem.getText());
        contentsItem.addActionListener(mainProg);

        tWHelpMenu.add(troubleshootingItem = new JMenuItem(RB("reportABug"), new ImageIcon(JMainMenu.class.getResource("icons/22/contents.png"))));
        troubleshootingItem.addActionListener(mainProg);
        tWHelpMenu.add(tfKeysItem = new JMenuItem(RB("tinyFugueKeyCodes"), new ImageIcon(JMainMenu.class.getResource("icons/22/contents.png"))));
        tfKeysItem.addActionListener(mainProg);


        // tWHelpMenu.add(troubleshootingItem = new JMenuItem(RB("troubleshooting")));
        tWHelpMenu.addSeparator();
        tWHelpMenu.add(aboutJamochaMUDItem = new JMenuItem(RB("aboutJamochaMUD"), new ImageIcon(JMainMenu.class.getResource("kehza.gif"))));
        aboutJamochaMUDItem.addActionListener(mainProg);
        tWHelpMenu.addActionListener(mainProg);

    }

    public synchronized void setCloseMUEnabled(boolean state) {
        closeMU.setEnabled(state);
    }

    public void removeAllPlugins() {
        tWPlugInMenu.removeAll();
        // tWPlugInMenu.add(managePlugIn);
        tWPlugInMenu.add(installPlugIn);
        tWPlugInMenu.add(removePlugIn);
        tWPlugInMenu.addSeparator();
    }

    // public void addPlugin(String name, boolean state) {
    public void addPlugin(anecho.JamochaMUD.plugins.PlugInterface plug) {

        if (DEBUG) {
            System.err.println("JMainMenu.addPlugin: " + plug);
            System.err.println("JMainMenu adding plug-in: " + plug.PlugInName());
        }
        /*
        JMenuItem tempName = new JMenuItem(name);
        // JCheckBoxMenuItem tempName = new JCheckBoxMenuItem(name);
        tempName.setEnabled(state);

        tempName.setActionCommand("plugin:" + name);
        tempName.addActionListener(settings.getMainWindowVariable());
        // tWPlugInMenu.add(name);
        tWPlugInMenu.add(tempName);
        */

        // JMenuItem tempName = new JMenuItem(name);
        String name = plug.PlugInName();
        boolean state = plug.isActive();
        boolean properties = plug.HasProperties();

        JMenu tempName = new JMenu(name);
        if (state) {
            tempName.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/Sphere_Green.png")));
        } else {
            tempName.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/Sphere_Red.png")));
        }

        //tempName.setActionCommand("plugin:" + name);
        //tempName.addActionListener(settings.getMainWindowVariable());
        JMenuItem prop = new JMenuItem(RB("properties"));
        prop.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/configure.png")));

        if (properties) {
            prop.setActionCommand("plugin:" + name);
            prop.addActionListener(settings.getMainWindowVariable());
        } else {
            // No settings are available for this item
            prop.setEnabled(false);
        }

        JMenuItem action = new JMenuItem();
        // Check to see if we should offer the enable or disable menu item
        if (state) {
            action.setText(RB("disable"));
            action.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/button_cancel.png")));
            action.setActionCommand("disable:" + name);
        } else {
            action.setText(RB("enable"));
            action.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/button_ok.png")));
            action.setActionCommand("enable:" + name);
        }

        action.addActionListener(settings.getMainWindowVariable());

        JMenuItem desc = new JMenuItem(RB("description"));
        desc.setActionCommand("description:" + name);
        desc.addActionListener(settings.getMainWindowVariable());

        tempName.add(prop);
        tempName.add(action);
        tempName.addSeparator();
        tempName.add(desc);

        tWPlugInMenu.add(tempName);

    }

    /** This method allows the plug-in and menu items to be updated to reflect
     * the current state of the plug-in.
     * @param state     the new state of our plug-in
     * @param actCom    the action-command we use to locate our menu item
     * true means the plug-in is active
     * false means the plug-in is inactive
     */
    public void changePlugInState(boolean state, String actCom) {
        if (DEBUG) {
            System.err.println("JMainMenu.changePlugInState() called.");
        }

        /*
        if (state) {
            action.setText(RB("disable"));
            action.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/button_cancel.png")));
            action.setActionCommand("disable:" + name);
        } else {
            action.setText(RB("enable"));
            action.setIcon(new ImageIcon(JMainMenu.class.getResource("icons/22/button_ok.png")));
            action.setActionCommand("enable:" + name);
            }
            */

    }

    public void updateConnectionMenu(MuckMain mainProg) {
        MuckMain mainVar = settings.getMainWindowVariable();

        // First, check to see if we have any connections.
        // If not, then we don't need to do any more here
        if (DEBUG) {
            System.err.println("JMainMenu.updateConnectionMenu: checking total connections");
        }

        CHandler connHandler = settings.getConnectionHandler();
        int totalCon = connHandler.totalConnections();

        if (DEBUG) {
            System.err.println("JMainMenu.updateConnectionMenu: total connections successful");
        }

        // if (connHandler.totalConnections() < 1) {
        if (totalCon < 1) {
            return;
        }


        // Clear the current list of MU*'s
        tWMUListMenu.removeActionListener(mainVar);
        tWMUListMenu.removeAll();

        // Add the two generic entries... the next and previous choices
        StringBuffer tempName;
        JMenuItem tempMenu;
        // MenuShortcut shortCut;

        tempMenu = new JMenuItem(RB("previousMU"), new ImageIcon(JMainMenu.class.getResource("icons/22/1leftarrow.png")));
        // coloursItem.setMnemonic(KeyEvent.VK_F);
        tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, ActionEvent.CTRL_MASK));
        // shortCut = new MenuShortcut(KeyEvent.VK_PAGE_DOWN, false);
        // tempMenu.setShortcut(shortCut);
        tempMenu.setActionCommand(RB("previousMU"));
        tempMenu.addActionListener(mainProg);

        if (DEBUG) {
            System.err.println("JMainMenu.updateConnectionMenu: checking total connections.");
        }


        // if (connHandler.totalConnections() < 2) {
        if (totalCon < 2) {
            tempMenu.setEnabled(false);
        }

        if (DEBUG) {
            System.err.println("JmainMenu.updateConnectionMenu: completd checking total connections.");
        }

        tWMUListMenu.add(tempMenu);

        tempMenu = new JMenuItem(RB("nextMU"), new ImageIcon(JMainMenu.class.getResource("icons/22/1rightarrow.png")));
        tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, ActionEvent.CTRL_MASK));
        // shortCut = new MenuShortcut(KeyEvent.VK_PAGE_UP, false);
        // tempMenu.setShortcut(shortCut);
        tempMenu.setActionCommand(RB("nextMU"));
        tempMenu.addActionListener(mainProg);
        // if (connHandler.totalConnections() < 2) {
        if (totalCon < 2) {
            tempMenu.setEnabled(false);
        }
        tWMUListMenu.add(tempMenu);

        tWMUListMenu.addSeparator();

        // int tc = connHandler.totalConnections() - 1;
        int tc = totalCon - 1;
        int activeConnection = connHandler.getActiveMUIndex();

        // Loop through our connections and build a new menu
        for (int i = 0; i <= tc; i++) {
            if (DEBUG) {
                System.err.println("JMainMenu: Getting muck name for menu.");
            }

            tempName = new StringBuffer(connHandler.getTitle(i));

            if (DEBUG) {
                System.err.println("JMainMenu: Menu name: " + tempName);
            }

            if (i == activeConnection) {
                // This is our active MU*
                // We'll just differentiate with a check beside it's name
                tWMUListMenu.add(new JCheckBoxMenuItem(tempName.toString(), true));
            } else {
                tempMenu = new JMenuItem(tempName.toString());
                tempMenu.addActionListener(mainProg);

                switch(i) {
                case 0:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_1, false);
                    break;
                case 1:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_2, false);
                    break;
                case 2:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_3, false);
                    break;
                case 3:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_4, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_4, false);
                    break;
                case 4:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_5, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_5, false);
                    break;
                case 5:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_6, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_6, false);
                    break;
                case 6:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_7, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_7, false);
                    break;
                case 7:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_8, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_8, false);
                    break;
                case 8:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_9, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_9, false);
                    break;
                case 9:
                    tempMenu.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_0, ActionEvent.CTRL_MASK));
                    // shortCut = new MenuShortcut(KeyEvent.VK_0, false);
                    break;
                }

                // tempMenu.setShortcut(shortCut);

                tempMenu.setActionCommand("ChangeMU:" + i);
                tempMenu.addActionListener(mainVar);
                tWMUListMenu.add(tempMenu);
            }
        }

        tWMUListMenu.addActionListener(mainVar);

    }

    /** Set the menus to reflect if we're connected (true) or not (false) */
    public void setConnected(boolean state) {
        // This changes the flags on the MuckMain menu to 'connected'
        // Connect to MU*
        cTM.setEnabled(true);
        // Disconnect from MU*
        dFM.setEnabled(state);
        // Reconnect to MU*
        rTM.setEnabled(!state);
    }

    public boolean isAutoFocus() {
        return tWAutoFocus.getState();
    }

    public boolean isReleasePause() {
        return tWReleasePauseItem.getState();
    }

    private String RB(String itemTarget) {
        return reader.LangString(anecho.JamochaMUD.JMConfig.BUNDLEBASE, itemTarget);
    }

    /*
    // private JMenuItem buildMenuItem(String itemTitle, String iconName, KeyEvent mneKey, KeyEvent AccKey, boolean evtMask) {
    private JMenuItem buildMenuItem(String itemTitle, String iconName, int mneKey, boolean hasAcc, int accKey, boolean evtMask) {
        tempMenuItem = new JMenuItem(RB(itemTitle));

        if (iconName != null) {
            tempMenuItem.setIcon(new ImageIcon(JMainMenu.class.getResource(iconName)));
        }

        tempMenuItem.setMnemonic(mneKey);
        if (hasAcc) {
            if (evtMask) {
                tempMenuItem.setAccelerator(KeyStroke.getKeyStroke(accKey, ActionEvent.CTRL_MASK));
            } else {
                tempMenuItem.setAccelerator(KeyStroke.getKeyStroke(accKey, 0));
            }
        }

        return tempMenuItem;
        }
        */
}