 /**
  * PlugInstaller allows JamochaMUD users a convenient way to
  * add and remove plug-ins from JamochaMUD.
  * $Id: PlugInInstaller.java,v 1.1 2004/02/01 03:29:53 jeffnik Exp $
  */

/* JamochaMUD, a Muck/Mud client program
 * Copyright (C) 1998-2004 Jeff Robinson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package anecho.JamochaMUD;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.*;

public class PlugInInstaller implements ActionListener {

    JMConfig settings;
    boolean userDir = false;
    boolean masterDir = false;
    boolean useSwing = false;
    boolean msgResponse = false; // Return message from AWT dialogue
    Dialog msg;

    String masterPath, userPath;
    int BUFFER = 2048;
    // FileOutputStream outStream;
    // BufferedOutputStream buffOutStream;

    public PlugInInstaller(JMConfig mainSettings) {
        settings = mainSettings;
        if (settings.getJMboolean(JMConfig.USESWING)) {
            useSwing = true;
        }
    }

    /** Display a file dialogue so a user may install a new plug-in.
     * To make things easier, we will require JamochaMUD plug-ins to
     * be packaged in plain old .zip files, but bearing the extension
     * .jpa (JamochaMUD Plug-in Archive).  This zipfile will contain
     * a MANIFEST file listing, and example of which follows with
     * comments in brackets (only NAME and PLUGCLASS are REQUIRED):
     * NAME: Test Plug-in (Name of the plug-in)
     * VERSION: 1.0 (Version of the plug-in)
     * PLUGCLASS: TestPlugIn (Name of the main Java class of the plug-in)
     * JVM: 1.1 (Minimum Java Virtual Machine needed to use the plug-in)
     * JMUDVER: 04-01-12 (Minimum version of JamochaMUD needed... numeric portion only)
     * LICENSE: TestPlugInDir\COPYING (location in the .jpa archive of the license file)
     * README: TestPlugInDir\readme.txt (any special instructions for the user)
     */
    public void install() {
        Frame parent = settings.getJMFrame(JMConfig.MUCKMAINFRAME);
        FileDialog installPath = new FileDialog(parent, "Choose .jpa to install", FileDialog.LOAD);
        installPath.show();

        masterPath = settings.getJMString(JMConfig.MASTERPLUGINDIR);
        userPath = settings.getJMString(JMConfig.USERPLUGINDIR);

        if (installPath.getFile() != null) {
            String plugPath = installPath.getDirectory() + installPath.getFile();

            // Select location(s) for plug-in installation
            if (!selectTargets()) {
                // The user didn't select a directory so we'll leave
                return;
            }

            if (!unpackPlugIn(plugPath)) {
                return;
            }

            // if (settings.getJMboolean(JMConfig.USESWING)) {
            if (useSwing) {
                javax.swing.JOptionPane.showMessageDialog(settings.getJMFrame(JMConfig.MUCKMAINFRAME),
                                                          "Plug-in installed successfully.\nYou may need to restart JamochaMUD\nto use the plug-in.");
            } else {
                // anecho.gui.OKBox successMsg = new anecho.gui.OKBox(settings.getJMFrame(JMConfig.MUCKMAINFRAME), "Success!", true, "Plug-in installed successfully.\nYou may need to restart JamochaMUD\nto use the plug-in.");
                anecho.gui.OKBox successMsg = new anecho.gui.OKBox(settings.getJMFrame(JMConfig.MUCKMAINFRAME), "Success!", true);
                // successMsg.setText("Plug-in installed successfully.\nYou may need to restart JamochaMUD\nto use the plug-in.");
                successMsg.setText("Plug-in installed successfully.  You may need to restart JamochaMUD to use the plug-in.");
                successMsg.show();
            }
        }
    }

    /** unpack and install the given plug-in */
    private boolean unpackPlugIn(String plugLocation) {
        boolean unpackResult = false;
        ZipFile plugPack;
        ZipEntry entry, testEntry;
        Vector entryList = new Vector();
        // FileInputStream inStream;
        InputStream inStream;
        String testDestName;

        try {
            plugPack = new ZipFile(plugLocation);
        } catch (Exception foo) {
            System.out.println("Exception opening " + plugLocation);
            return unpackResult;
        }

        Enumeration plugList = plugPack.entries();

        while (plugList.hasMoreElements()) {
            // grab a zip file entry
            entry = (ZipEntry)plugList.nextElement();
            entryList.addElement(entry);
            // System.out.println(entry.getName());
        }

        int totalEntries = entryList.size();

        System.out.println("There are " + totalEntries + " entries in our plugPack.");

        // Look for our MANIFEST file
        if (hasManifest(entryList)) {
            System.out.println("We found the manifest file!");
        } else {
            System.out.println("No Manifest file was found, cannot load.");
            return unpackResult;
        }

        File testFile;
        String pathSep = System.getProperty("file.separator");

        BufferedOutputStream dest = null;
        // FileInputStream fis = new FileInputStream();
        
        // Open up a stream to the plug-in archive
        // inStream = zipFile.getInputStream(plugLocation);
        // inStream = plugPack.getInputStream(plugLocation);

        for (int i = 0; i < totalEntries; i++) {
            testEntry = (ZipEntry)entryList.elementAt(i);

            if (testEntry.isDirectory()) {
                // Create a directory first
                // System.out.println("Creating directory: " + testEntry.getName());
                // System.out.println("at: " + userPath);
                // System.out.println("at: " + masterPath);
                // This is not robust, just for demonstration purposes.

                if (userDir) {
                    testFile = new File(userPath + pathSep + testEntry.getName());
                    System.out.println(userPath + pathSep + testEntry.getName());
                    testFile.mkdirs();
                    continue;
                }
                if (masterDir) {
                    testFile = new File(masterPath + pathSep + testEntry.getName());
                    System.out.println(masterPath + pathSep + testEntry.getName());
                    testFile.mkdirs();
                    continue;
                }
            } else {
                if (!testEntry.getName().equalsIgnoreCase("manifest")) {
                    // Uncompress the file
                    try {
                        // System.out.println("Creating file: " + testEntry.getName());
                        // System.out.println("at: " + userPath);
                        // System.out.println("at: " + masterPath);

                        // System.out.println("Creating FileInputStream.");
                        inStream = plugPack.getInputStream(testEntry);
                        // System.out.println("Creating ZipInputStream.");
                        ZipInputStream zis = new ZipInputStream(new BufferedInputStream(inStream));


                        if (userDir) {
                            // outStream = userPath + pathSep + testEntry.getName();
                            // testFile = new File(userPath + pathSep + testEntry.getName());
                            // testFile.mkdirs();
                            // outStream = new FileOutputStream(userPath + pathSep + testEntry.getName());
                            // buffOutStream = new BufferedOutputStream(outStream, BUFFER);                }
                            // unzipMethod(userPath + pathSep, testEntry);
                            // System.out.println("Extracting: " + testEntry);
                            int count;
                            byte data[] = new byte[BUFFER];
                            // write the files to the disk
                            FileOutputStream fos;

                            if (userDir) {
                                testDestName = userPath + pathSep + testEntry.getName();
                                // fos = new FileOutputStream(userPath + pathSep + testEntry.getName());
                                fos = new FileOutputStream(testDestName);
                            } else {
                                testDestName = masterPath + pathSep + testEntry.getName();
                                fos = new FileOutputStream(testDestName);
                            }

                            dest = new BufferedOutputStream(fos, BUFFER);
                            while ((count = zis.read(data, 0, BUFFER))
                                   != -1) {
                                dest.write(data, 0, count);
                            }
                            dest.flush();
                            dest.close();
                        }
                    } catch (Exception ze) {
                        System.out.println("ZipInputStream exception " + ze);
                        continue;
                    }
                }

            }
        }

        try {
            plugPack.close();
        } catch (Exception e) {
            System.out.println("Exception while closing plugPack: " + e);
        }

        unpackResult = true;

        return unpackResult;
    }

    /** Check the given entryList for a manifest file, returning &quot;true" if one exists */
    private boolean hasManifest(Vector entryList) {
        boolean result = false;
        int entries = entryList.size();

        for (int i = 0; i < entries; i++) {
            if (entryList.elementAt(i).toString().equalsIgnoreCase("manifest")) {
                result = true;
            }
        }

        return result;
    }

    /** Create a dialogue and ask the user which directories to install to */
    private boolean selectTargets() {
        boolean selectResult = false;

        // GridbagLayout layout = new GridbagLayout();
        // if (settings.getJMboolean(JMConfig.USESWING)) {
        if (useSwing) {
            javax.swing.JCheckBox userPathBox = new javax.swing.JCheckBox(userPath, true);
            javax.swing.JCheckBox masterPathBox = new javax.swing.JCheckBox(masterPath, false);
            javax.swing.JPanel choicePanel = new javax.swing.JPanel(new GridLayout(3, 1));
            javax.swing.JLabel choiceDesc = new javax.swing.JLabel("Choose the directories that you wish to have the plug-in installed to.");

            choicePanel.add(choiceDesc);
            choicePanel.add(userPathBox);
            choicePanel.add(masterPathBox);

            // javax.swing.JOptionPane optionPane = new javax.swing.JOptionPane(choicePanel, javax.swing.JOptionPane.OK_CANCEL_OPTION);
            int response = javax.swing.JOptionPane.showOptionDialog(settings.getJMFrame(JMConfig.MUCKMAINFRAME),
                                                                    choicePanel,
                                                                    "Choose installation location(s)",
                                                                    javax.swing.JOptionPane.OK_CANCEL_OPTION,
                                                                    javax.swing.JOptionPane.QUESTION_MESSAGE,
                                                                    null,
                                                                    null,
                                                                    null);

            if (response == javax.swing.JOptionPane.OK_OPTION) {
                selectResult = true;
                if (userPathBox.isSelected()) {
                    userDir = true;
                }
                if (masterPathBox.isSelected()) {
                    masterDir = true;
                }
            }
            // optionPane.setVisible(true);

        } else {
            Checkbox userPathBox = new Checkbox(userPath, true);
            Checkbox masterPathBox = new Checkbox(masterPath, false);

            Button okayButton = new Button("Okay");
            Button cancelButton = new Button("Cancel");
            okayButton.setActionCommand("OKAY");
            okayButton.addActionListener(this);
            cancelButton.setActionCommand("CANCEL");
            cancelButton.addActionListener(this);

            Label choiceDesc = new Label("Choose the directories that you wish to have the plug-in installed to.");

            Panel choicePanel = new Panel(new GridLayout(4, 1));
            Panel buttonPanel = new Panel(new GridLayout(1, 2));

            buttonPanel.add(okayButton);
            buttonPanel.add(cancelButton);


            choicePanel.add(choiceDesc);
            choicePanel.add(userPathBox);
            choicePanel.add(masterPathBox);
            choicePanel.add(buttonPanel);

            msg = new Dialog(settings.getJMFrame(JMConfig.MUCKMAINFRAME),
                                    "Choose installation location(s)",
                                    true);

            msg.add(choicePanel);
            msg.pack();
            msg.setVisible(true);

            if (msgResponse) {
                selectResult = true;
                if (userPathBox.getState()) {
                    userDir = true;
                }
                if (masterPathBox.getState()) {
                    masterDir = true;
                }
            }

        }

        return selectResult;
    }

    /** This method is used solely for the AWT dialogue */
    public void actionPerformed(ActionEvent buttonEvent) {
        // System.out.println("Event: " + buttonEvent);
        if (buttonEvent.getActionCommand().equals("OKAY")) {
            msgResponse = true;
        }

        msg.setVisible(false);
        msg.dispose();
    }

        /**
    private void unzipMethod(String destPath, Vector entryList) {
        FileOutputStream outStream;
        BufferedOutputStream buffOutStream;
        while ((count = zin.read(data, 0, BUFFER)) != -1) {
            System.out.println("Extracting: " + entryName);
            outStream = new FileOutputStream(destPath + pathSep + entryName.getName());
            buffOutStream = new BufferedOutputStream(outStream, BUFFER);
            int count;
            byte data[] = new byte[BUFFER];
            //System.out.write(x);
            buffOutStream.write(data, 0, count);
        }
    }
    */

    public void remove() {
    }


}